@@ -522,7 +522,10 @@ def get_subset_limits(subsetlimits, parser):
         # or
         # <dim_name>=[<start_value>]
         match = re.match(r'(?P<dim>[^=]+)=\[(?P<start>[^],]+)(?:,(?P<end>[^],]+))?\]$', seg)
-        if match is None or match.group('dim') is None or match.group('start') is None:
+        if seg.startswith('shape'):
+            # Don't use the regexp for this as it gets confused with the commas
+            limit_dict['shape'] = seg.split('=')[1]
+        elif match is None or match.group('dim') is None or match.group('start') is None:
             parser.error(
                 "A dimension for subsetting does not have dimension name, start value and/or end value specified")
         else:
@@ -629,7 +632,7 @@ def parse_colon_and_comma_separated_arguments(inputs, parser, options, compulsor
     return input_dicts
 
 
-def split_outside_brackets(input, seps=[','], brackets={'[': ']'}):
+def split_outside_brackets(input, seps=[','], brackets={'[': ']', '(': ')'}):
     """Splits an input string at separators that are not within brackets.
     :param input: input string to parse
     :param seps: list of separator characters - default: comma
@@ -953,7 +956,7 @@ def validate_version_args(arguments, parser):
 
 validators = {'plot': validate_plot_args,
               'info': validate_info_args,
-              'col': validate_col_args,
+              'collocate': validate_col_args,
               'aggregate': validate_aggregate_args,
               'collapse': validate_collapse_args,
               'subset': validate_subset_args,
@@ -961,6 +964,10 @@ validators = {'plot': validate_plot_args,
               'stats': validate_stats_args,
               'version': validate_version_args}
 
+aliases = {'col': 'collocate',
+           'sub': 'subset',
+           'agg': 'aggregate'}
+
 
 def parse_args(arguments=None):
     """
@@ -972,6 +979,10 @@ def parse_args(arguments=None):
         # sys.argv[0] is the name of the script itself
         arguments = sys.argv[1:]
     main_args = parser.parse_args(arguments)
+    # TODO I don't really like this as I have to specify the aliases twice...
+    if main_args.command in aliases.keys():
+        main_args.command = aliases[main_args.command]
+
     # Firstly deal with logging verbosity - in case we log anything in the validation
     # The 'screen' handler is the first in the list
     if main_args.quiet:
